@@ -1,4 +1,5 @@
-#TODO: Make work with standardized parameter estimates
+# TODO: Make work with standardized parameter estimates
+
 compareModels <- function(m1, m2, show="all", equalityMargin=c(param=0.0001, pvalue=0.0001), sort="none", showFixed=FALSE, showNS=TRUE, diffTest=FALSE) {
   #options for show:
   # -all: print equal params, unequal params, and unique params
@@ -10,7 +11,6 @@ compareModels <- function(m1, m2, show="all", equalityMargin=c(param=0.0001, pva
   # -allsummaries  
   
   #defaults to show equal and unequal params
-  require(reshape)
   require(plyr)
   
   #if a single scalar is passed in with no name, then use that value for param and pvalue diffs
@@ -51,7 +51,7 @@ compareModels <- function(m1, m2, show="all", equalityMargin=c(param=0.0001, pva
     
     if (!"allsummaries" %in% show) {
       comparators <- c("Title", "Observations", "Estimator", "Parameters", "LL", "AIC", "BIC", "ChiSqM_Value", "ChiSqM_DF",
-          "CFI", "TLI", "RMSEA", "SRMR", "WRMR")
+          "CFI", "TLI", "RMSEA_Estimate", "SRMR", "WRMR")
       
       sumCombined <- sumCombined[,intersect(comparators, names(sumCombined))]      
     }
@@ -85,52 +85,72 @@ compareModels <- function(m1, m2, show="all", equalityMargin=c(param=0.0001, pva
     print(wrappedOutput, quote=FALSE)
     
     #chi-square difference testing
-    if (diffTest == TRUE && m1Summaries$Estimator %in% c("ML", "MLM", "MLR", "WLS", "WLSM") && m1Summaries$Parameters != m2Summaries$Parameters) {
-      if (m1Summaries$Estimator == m2Summaries$Estimator ) {
-        if (m1Summaries$Parameters < m2Summaries$Parameters) H0m <- m1Summaries
-        else H0m <- m2Summaries
-        
-        if (m1Summaries$Parameters > m2Summaries$Parameters) H1m <- m1Summaries
-        else H1m <- m2Summaries
-        
-        if (m1Summaries$Estimator == "MLR") {
-          #Chi-square difference test for MLR models based on loglikelihood
-          cd <- (H0m$Parameters*H0m$LLCorrectionFactor - H1m$Parameters*H1m$LLCorrectionFactor)/(H0m$Parameters - H1m$Parameters)
-          
-          ChiSqDiff <- -2*(H0m$LL - H1m$LL)/cd
-          dfDiff <- (H0m$ChiSqM_DF - H1m$ChiSqM_DF)
-          
-          cat("\n  MLR Chi-Square Difference test for nested models based on loglikelihood\n  -----------------------------------------------------------------------\n\n")
-          cat("  Difference Test Scaling Correction: ", cd, "\n")
-          cat("  Chi-square difference: ", round(ChiSqDiff, 4), "\n")
-          cat("  Diff degrees of freedom: ", dfDiff, "\n")
-          cat("  P-value: ", round(pchisq(ChiSqDiff, dfDiff, lower.tail=FALSE), 4), "\n")
-          cat("\n  Note: The chi-square difference test assumes that these models are nested.\n  It is up to you to verify this assumption.\n")
+    if (diffTest == TRUE) {
+      if ("ChiSqDiffTest_Value" %in% unique(c(names(m1Summaries), names(m2Summaries)))) {
+        #computed using difftest
+        if ("ChiSqDiffTest_Value" %in% names(m1Summaries)) {
+          val <- m1Summaries$ChiSqDiffTest_Value
+          dfDiff <- m1Summaries$ChiSqDiffTest_DF
+          pval <- m1Summaries$ChiSqDiffTest_PValue
+        } else {
+          val <- m2Summaries$ChiSqDiffTest_Value
+          dfDiff <- m2Summaries$ChiSqDiffTest_DF
+          pval <- m2Summaries$ChiSqDiffTest_PValue
         }
-        if (m1Summaries$Estimator %in% c("ML", "MLR", "MLM", "WLSM")) {
-          if (m1Summaries$Estimator %in% c("ML", "WLS"))
-            #for ML and WLS,  no need to correct chi-square with scaling factors
-            ChiSqDiff <- H0m$ChiSqM_Value - H1m$ChiSqM_Value
-          else {
-            cd <- (H0m$ChiSqM_DF*H0m$ChiSqM_ScalingCorrection - H1m$ChiSqM_DF*H1m$ChiSqM_ScalingCorrection)/(H0m$ChiSqM_DF - H1m$ChiSqM_DF)
-            ChiSqDiff <- (H0m$ChiSqM_Value*H0m$ChiSqM_ScalingCorrection - H1m$ChiSqM_Value*H1m$ChiSqM_ScalingCorrection)/cd            
+        
+        cat("\n  Chi-Square Difference Test for Nested Models using DIFFTEST in Mplus\n  -----------------------------------------------------------------------\n\n")
+        cat("  Chi-square difference: ", round(val, 4), "\n")
+        cat("  Diff degrees of freedom: ", dfDiff, "\n")
+        cat("  P-value: ", round(pval, 4), "\n")
+        
+      } else if (m1Summaries$Estimator %in% c("ML", "MLM", "MLR", "WLS", "WLSM") && m1Summaries$Parameters != m2Summaries$Parameters) {
+        if (m1Summaries$Estimator == m2Summaries$Estimator ) {
+          if (m1Summaries$Parameters < m2Summaries$Parameters) H0m <- m1Summaries
+          else H0m <- m2Summaries
+          
+          if (m1Summaries$Parameters > m2Summaries$Parameters) H1m <- m1Summaries
+          else H1m <- m2Summaries
+          
+          if (m1Summaries$Estimator == "MLR") {
+            #Chi-square difference test for MLR models based on loglikelihood
+            cd <- (H0m$Parameters*H0m$LLCorrectionFactor - H1m$Parameters*H1m$LLCorrectionFactor)/(H0m$Parameters - H1m$Parameters)
+            
+            ChiSqDiff <- -2*(H0m$LL - H1m$LL)/cd
+            dfDiff <- (H0m$ChiSqM_DF - H1m$ChiSqM_DF)
+            
+            cat("\n  MLR Chi-Square Difference Test for Nested Models Based on Loglikelihood\n  -----------------------------------------------------------------------\n\n")
+            cat("  Difference Test Scaling Correction: ", cd, "\n")
+            cat("  Chi-square difference: ", round(ChiSqDiff, 4), "\n")
+            cat("  Diff degrees of freedom: ", dfDiff, "\n")
+            cat("  P-value: ", round(pchisq(ChiSqDiff, dfDiff, lower.tail=FALSE), 4), "\n")
+            cat("\n  Note: The chi-square difference test assumes that these models are nested.\n  It is up to you to verify this assumption.\n")
           }
-          
-          dfDiff <- (H0m$ChiSqM_DF - H1m$ChiSqM_DF)
-          
-          cat("\n  ", m1Summaries$Estimator, " Chi-Square Difference test for nested models\n  --------------------------------------------\n\n", sep="")
-          if (! m1Summaries$Estimator %in% c("WLS", "ML")) cat("  Difference Test Scaling Correction:", cd, "\n")
-          cat("  Chi-square difference:", round(ChiSqDiff, 4), "\n")
-          cat("  Diff degrees of freedom:", dfDiff, "\n")
-          cat("  P-value:", round(pchisq(ChiSqDiff, dfDiff, lower.tail=FALSE), 4), "\n")
-          cat("\nNote: The chi-square difference test assumes that these models are nested.\n  It is up to you to verify this assumption.\n")          
-        }
+          if (m1Summaries$Estimator %in% c("ML", "MLR", "MLM", "WLSM")) {
+            if (m1Summaries$Estimator %in% c("ML", "WLS"))
+              #for ML and WLS,  no need to correct chi-square with scaling factors
+              ChiSqDiff <- H0m$ChiSqM_Value - H1m$ChiSqM_Value
+            else {
+              cd <- (H0m$ChiSqM_DF*H0m$ChiSqM_ScalingCorrection - H1m$ChiSqM_DF*H1m$ChiSqM_ScalingCorrection)/(H0m$ChiSqM_DF - H1m$ChiSqM_DF)
+              ChiSqDiff <- (H0m$ChiSqM_Value*H0m$ChiSqM_ScalingCorrection - H1m$ChiSqM_Value*H1m$ChiSqM_ScalingCorrection)/cd            
+            }
+            
+            dfDiff <- (H0m$ChiSqM_DF - H1m$ChiSqM_DF)
+            
+            cat("\n  ", m1Summaries$Estimator, " Chi-Square Difference test for nested models\n  --------------------------------------------\n\n", sep="")
+            if (! m1Summaries$Estimator %in% c("WLS", "ML")) cat("  Difference Test Scaling Correction:", cd, "\n")
+            cat("  Chi-square difference:", round(ChiSqDiff, 4), "\n")
+            cat("  Diff degrees of freedom:", dfDiff, "\n")
+            cat("  P-value:", round(pchisq(ChiSqDiff, dfDiff, lower.tail=FALSE), 4), "\n")
+            cat("\nNote: The chi-square difference test assumes that these models are nested.\n  It is up to you to verify this assumption.\n")          
+          }
+        } else
+          warning("Cannot compute chi-square difference test because different estimators used: ", m1Summaries$Estimator, " and ", m2Summaries$Estimator)
       }
-      else
-        warning("Cannot compute chi-square difference test because different estimators used: ", m1Summaries$Estimator, " and ", m2Summaries$Estimator)
-    }
+    } 
   }
   
+  
+  ###PARAMETER COMPARISON
   #will blow up in cases where std output with no est_se or pval
   
   #fuzzy equality
@@ -155,38 +175,58 @@ compareModels <- function(m1, m2, show="all", equalityMargin=c(param=0.0001, pva
     if (inherits(m2, "mplus.model")) m2Params <- m2$parameters$unstandardized
     else m2Params <- m2
     
-    #match up paramheader and param combinations
-    m1Params$paramCombined <- paste(m1Params$paramHeader, m1Params$param, sep=".")
-    m2Params$paramCombined <- paste(m2Params$paramHeader, m2Params$param, sep=".")
-    
-    #identify columns present in both data.frames
-    matchCols <- intersect(m1Params$paramCombined, m2Params$paramCombined)
-    m1Only <- setdiff(m1Params$paramCombined, m2Params$paramCombined)
-    m2Only <- setdiff(m2Params$paramCombined, m1Params$paramCombined)
-    
-    m1Params <- reshape::rename(m1Params, c(est="m1_est", se="m1_se", est_se="m1_est_se", pval="m1_pval"))
-    m2Params <- reshape::rename(m2Params, c(est="m2_est", se="m2_se", est_se="m2_est_se", pval="m2_pval"))
-    
-    matchDF <- merge(m1Params[m1Params$paramCombined %in% matchCols,], m2Params[m2Params$paramCombined %in% matchCols,], by=c("paramHeader", "param"), all.x=TRUE, all.y=TRUE)
-    matchDF <- subset(matchDF, select=c(-paramCombined.x, -paramCombined.y)) #just retain paramHeader and param
-    
-    #remove fixed parameters (only if fixed in both models)
-    if (!showFixed) {
-      matchDF <- subset(matchDF, !(m1_est_se == 999.000 & m2_est_se == 999.000))
-      #N.B. I'm dubious about removing fixed from m1/m2 params because then the "unique" output omits these
-      m1Params <- subset(m1Params, !m1_est_se == 999.000)
-      m2Params <- subset(m2Params, !m2_est_se == 999.000)
-    }
-    
-    #remove non-significant effects, if requested
-    if (is.logical(showNS) && showNS==FALSE) showNS <- .05 #default alpha for filtering N.S. results
-    if (!showNS==TRUE) {
-      matchDF <- subset(matchDF, !(m1_pval > showNS & m2_pval > showNS))
-      m1Params <- subset(m1Params, !(m1_pval > showNS))
-      m2Params <- subset(m2Params, !(m2_pval > showNS))
-    }
-    
-    if (sort=="type") {
+	#check for multiple groups, latent classes, and multilevel output
+	uniqueParamCols <- c("paramHeader", "param")
+  
+  if ("Group" %in% names(m1Params)) {
+    uniqueParamCols <- c(uniqueParamCols, "Group")
+    multipleGroups <- TRUE
+  } else multipleGroups <- FALSE
+  
+  if ("LatentClass" %in% names(m1Params)) {
+    uniqueParamCols <- c(uniqueParamCols, "LatentClass")
+    latentClasses <- TRUE
+  } else latentClasses <- FALSE
+  
+  if ("BetweenWithin" %in% names(m1Params)) {
+    uniqueParamCols <- c(uniqueParamCols, "BetweenWithin")
+    multilevel <- TRUE
+  } else multilevel <- FALSE
+  
+  #match up paramheader and param combinations
+  m1Params$paramCombined <- apply(m1Params[,uniqueParamCols], 1, paste, collapse=".")
+  m2Params$paramCombined <- apply(m2Params[,uniqueParamCols], 1, paste, collapse=".")
+  
+  #identify columns present in both data.frames
+  matchCols <- intersect(m1Params$paramCombined, m2Params$paramCombined)
+  m1Only <- setdiff(m1Params$paramCombined, m2Params$paramCombined)
+  m2Only <- setdiff(m2Params$paramCombined, m1Params$paramCombined)
+  
+  m1Params <- plyr::rename(m1Params, c(est="m1_est", se="m1_se", est_se="m1_est_se", pval="m1_pval"))
+  m2Params <- plyr::rename(m2Params, c(est="m2_est", se="m2_se", est_se="m2_est_se", pval="m2_pval"))
+  
+  #perhaps merge on uniqueParamCols (ensures that any group, bw/wi, and latent classes match)
+  #matchDF <- merge(m1Params[m1Params$paramCombined %in% matchCols,], m2Params[m2Params$paramCombined %in% matchCols,], by=c("paramHeader", "param"), all.x=TRUE, all.y=TRUE)
+  matchDF <- merge(m1Params[m1Params$paramCombined %in% matchCols,], m2Params[m2Params$paramCombined %in% matchCols,], by=uniqueParamCols, all.x=TRUE, all.y=TRUE)
+  matchDF <- subset(matchDF, select=c(-paramCombined.x, -paramCombined.y)) #don't retain the matching column
+  
+  #remove fixed parameters (only if fixed in both models)
+  if (!showFixed) {
+    matchDF <- subset(matchDF, !(m1_est_se == 999.000 & m2_est_se == 999.000))
+    #N.B. I'm dubious about removing fixed from m1/m2 params because then the "unique" output omits these
+    m1Params <- subset(m1Params, !m1_est_se == 999.000)
+    m2Params <- subset(m2Params, !m2_est_se == 999.000)
+  }
+  
+  #remove non-significant effects, if requested
+  if (is.logical(showNS) && showNS==FALSE) showNS <- .05 #default alpha for filtering N.S. results
+  if (!showNS==TRUE) {
+    matchDF <- subset(matchDF, !(m1_pval > showNS & m2_pval > showNS))
+    m1Params <- subset(m1Params, !(m1_pval > showNS))
+    m2Params <- subset(m2Params, !(m2_pval > showNS))
+  }
+  
+  if (sort=="type") {
       #add a leading . to the replacement to filter on, by, and with to the top of the results output 
       paramType <- sapply(matchDF$paramHeader, sub, pattern=".*\\.(ON|WITH|BY)$", replacement=".\\1", ignore.case=TRUE, perl=TRUE)
       matchDF <- matchDF[order(paramType, matchDF$paramHeader, matchDF$param),]
@@ -196,8 +236,11 @@ compareModels <- function(m1, m2, show="all", equalityMargin=c(param=0.0001, pva
     }
     
     #order by each parameter match (could make optional)
-    matchDF <- matchDF[,c("paramHeader", "param", "m1_est", "m2_est", "m1_se", "m2_se", "m1_est_se", "m2_est_se", "m1_pval", "m2_pval")]
-    matchDF <- cbind(matchDF[,1:4], .=rep("|", nrow(matchDF)), matchDF[,5:6], .=rep("|", nrow(matchDF)), matchDF[,7:8], .=rep("|", nrow(matchDF)), matchDF[,9:10])
+    matchDF <- matchDF[,c(uniqueParamCols, "m1_est", "m2_est", "m1_se", "m2_se", "m1_est_se", "m2_est_se", "m1_pval", "m2_pval")]
+    matchDF <- cbind(matchDF[,c(uniqueParamCols, "m1_est", "m2_est")], .=rep("|", nrow(matchDF)), 
+        matchDF[,c("m1_se", "m2_se")], .=rep("|", nrow(matchDF)), 
+        matchDF[,c("m1_est_se", "m2_est_se")], .=rep("|", nrow(matchDF)), 
+        matchDF[,c("m1_pval", "m2_pval")])
     
     cat("\n=========\n\nModel parameter comparison\n--------------------------\n")
     paramsEqual <- with(matchDF, testEquality(m1_est, m2_est, equalityMargin["param"]))
@@ -281,62 +324,4 @@ compareModels <- function(m1, m2, show="all", equalityMargin=c(param=0.0001, pva
   }
   
   cat("\n==============\n")
-}
-
-
-#detritus from debacle of printing summaries side-by-side
-
-#doeesn't work with print at the moment
-#    if ("Title" %in% names(sumCombined))
-#      print(cbind(sapply(sumCombined$Title, function(title) {
-#            return(strwrap(title, width=25, exdent=2))
-#          }, USE.NAMES=FALSE)), quote=FALSE)
-#      sumCombined$Title <- sapply(sumCombined$Title, function(title) {
-#            return(paste(strwrap(title, width=25, exdent=2), collapse="\n"))
-#          })
-
-#      
-#    m1Printout <- capture.output(print(t(sumCombined[1,]), quote=FALSE))
-#    m2Printout <- capture.output(print(t(sumCombined[1,]), quote=FALSE))
-#      
-#    printout <- capture.output(print(t(sumCombined), quote=FALSE, width=10000))
-#    printout <- strwrap(encodeString(t(sumCombined), width=5000), width=30, exdent=2)
-#    
-#convert from two rows to two columns    
-#sumCombined[1,"Title"] <- "short title"
-
-#detritus from earlier effort
-#    m2Wrapped <- lapply(sumCombinedL, "[[", 2)
-#    
-#    sumCombined <- apply(sumCombined, c(1,2), strwrap, width=30, exdent=2)
-#    
-#    #this will pad lists where one is longer than the other
-#    sumCombined2 <- aaply(sumCombined, 2, function(col) {
-#          len1 <- length(col[[1]])
-#          len2 <- length(col[[2]])
-#          if (len1 > len2) col[[2]] <- c(col[[2]], rep("", len1-len2))
-#          else if (len2 > len1) col[[1]] <- c(col[[1]], rep("", len2-len1))
-#          return(lapply(col, cbind))
-#        })
-#    
-#    
-#    
-#    lapply(sumCombined2, function(el) {
-#          browser()
-#        })
-#    do.call("rbind", lapply(sumCombined2, "rbind"))
-#    #the above call converts to a list of the same dimension, but with varying length depending on wrap
-#    
-#    sumCombinedL <- lapply(sumCombined, function(col) {
-#          browser()
-#          len1 <- length(col[1])
-#          len2 <- length(col[2])
-#          if (len1 > len2) col[2] <- c(col[2], rep("", len1-len2))
-#          else if (len2 > len1) col[1] <- c(col[1], rep("", len2-len1))
-#          
-#        })
-#    sumCombined <- t(sumCombined)
-#    
-#    print(t(sumCombined[,!names(sumCombined)=="Title"]), quote=FALSE)
-#    
-#    cat(strwrap(capture.output(print(t(sumCombined), quote=FALSE)), width=30, exdent=2), sep="\n")
+}
\ No newline at end of file
